#  Copyright 2024 The Meatie Authors. All rights reserved.
#  Use of this source code is governed by a BSD-style license that can be found in the LICENSE file.

import importlib.metadata

__version__ = importlib.metadata.version("meatie")
